/*
Copyright (c) 2009 Steven M. Ottens

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

class SphericalMercator  {
  public static final double originShift =  2 * Math.PI * 6378137 / 2.0;
  private  SphericalMercator c_rd = null;
  /* singleton */
  public  SphericalMercator instance()
  {
    if (c_rd==null)
    {
      c_rd = new SphericalMercator();
    }
    return c_rd;
  }
       
  protected SphericalMercator() {
  }

  public float[] Project (double x, double y, float pminx, float pminy, float pmaxx, float pmaxy) {
    float result[] = new float[2];
    double[] pt = LLtoSM(x,y);
    double[] minpt = LLtoSM(minx,miny);
    double[] maxpt = LLtoSM(maxx,maxy);

    float mapHeight = height;
    float mapWidth = width;
    float ratio = mapHeight/mapWidth;
    float bboxWidth = abs((float)maxpt[0]-(float)minpt[0]);
    float bboxHeight = abs((float)maxpt[1]-(float)minpt[1]);
                
    if ((bboxWidth/width) > (bboxHeight/height)){
      mapWidth = bboxWidth;
      mapHeight = ratio*bboxWidth;
    }
    else if ((bboxWidth/width) < (bboxHeight/height)){
      mapHeight = bboxHeight;
      mapWidth = bboxHeight/ratio;
    } else {
      mapWidth = bboxWidth;
      mapHeight = bboxHeight;
    }
                
    pt[0] = pt[0] - minpt[0];
    pt[1] = pt[1] - minpt[1];
    pt[0] = (pt[0]/mapWidth)*(width) + (0.5*width)*(mapWidth-bboxWidth)/mapWidth; // 640;
    pt[1] = (pt[1]/mapHeight)*(height) +  (0.5*height)*(mapHeight-bboxHeight)/mapHeight; // 480;
    pt[1] = height - pt[1];
		
    for (int i=0; i<2; i++) {
      result[i] = (float) pt[i];
    }
    return result;
  }

//Based on http://www.maptiler.org/google-maps-coordinates-tile-bounpt-projection/
  private double[] LLtoSM(double x, double y) {
    double[] result = new double[2];
    result[0] = x * originShift / 180.0;
    double my = Math.log( Math.tan((90 + y) * Math.PI / 360.0 )) / (Math.PI / 180.0);
    result[1] = my * originShift / 180.0;                  
    return result;
  }	
}
